@@ -17,7 +17,7 @@ class CodeGenerator(object):
     """
 
     MAX_SIGNAL_STATES = 12
-    GENERATED_CODE_VERSION = "4.0"
+    GENERATED_CODE_VERSION = "5.x"
 
     def __init__(self, search_paths):
         self.search_paths = search_paths
@@ -95,7 +95,7 @@ class CodeGenerator(object):
         lines = []
         lines.append("const int MAX_MESSAGE_COUNT = %d;" %
                 self._max_message_count())
-        lines.append("CanMessage CAN_MESSAGES[][MAX_MESSAGE_COUNT] = {")
+        lines.append("CanMessageDefinition CAN_MESSAGES[][MAX_MESSAGE_COUNT] = {")
 
         def block(message_set):
             lines = []
@@ -161,11 +161,14 @@ class CodeGenerator(object):
             lines.append("        switch(address) {")
             for bus in message_set.valid_buses():
                 lines.append("        case %s:" % bus.controller)
-                lines.append("            *count = %d;" % len(
-                        list(bus.active_messages())))
-                for i, message in enumerate(bus.active_messages()):
-                    lines.append("            FILTERS[%d] = {%d, 0x%x, %d};" % (
-                            i, i, message.id, 1))
+                if bus.raw_can_mode == "unfiltered":
+                    lines.append("            *count = 0;")
+                else:
+                    lines.append("            *count = %d;" % len(
+                            list(bus.active_messages())))
+                    for i, message in enumerate(bus.active_messages()):
+                        lines.append("            FILTERS[%d] = {%d, 0x%x, %d};" % (
+                                i, i, message.id, 1))
                 lines.append("            break;")
             lines.append("        }")
             return lines
@@ -181,7 +184,7 @@ class CodeGenerator(object):
                 self.MAX_SIGNAL_STATES)
         lines.append("const int MAX_SIGNAL_COUNT = %d;" %
                 self._max_signal_count())
-        lines.append("CanSignalState SIGNAL_STATES[]"
+        lines.append("const CanSignalState SIGNAL_STATES[]"
                 "[MAX_SIGNAL_COUNT][MAX_SIGNAL_STATES] = {")
 
         def block(message_set, **kwargs):
@@ -310,13 +313,13 @@ class CodeGenerator(object):
                 for message in bus.active_messages():
                     lines.append(" " * 12 + "case 0x%x: // %s" % (message.id,
                             message.name))
-                    if message.handler is not None:
+                    for handler in message.handlers:
                         lines.append(" " * 16 + "%s(id, data, SIGNALS[%d], " % (
-                            message.handler, message_set.index) +
+                            handler, message_set.index) +
                                 "getSignalCount(), pipeline);")
                     for signal in sorted((s for s in message.signals.values()),
                             key=operator.attrgetter('generic_name')):
-                        if not signal.enabled:
+                        if not signal.enabled or signal.ignore:
                             continue
                         line = " " * 16
                         line += ("can::read::translateSignal(pipeline, "
@@ -329,16 +332,16 @@ class CodeGenerator(object):
                         lines.append(line)
                     lines.append("                break;")
                 lines.append("            }")
+                if (len(list(bus.active_messages())) == 0 or
+                        bus.raw_can_mode != "off"):
+                    lines.append(" " * 12 + "openxc::can::read::passthroughMessage("
+                            "bus, id, data, getMessages(), getMessageCount(), pipeline);")
                 lines.append("            break;")
             lines.append("        }")
             return lines
 
         lines.extend(self._message_set_switcher(block))
 
-        if self._max_message_count() == 0:
-            lines.append("    openxc::can::read::passthroughMessage(pipeline, "
-                    "id, data);")
-
         lines.append("}")
         lines.append("")
 
